
/*
    Convert the given string into its corresponding integer value using base 'base'.
    
    Maximum supported base is 16.
    
    return true on success.
*/
bool StringToInt(string &in s, int &out value, uint8 base)
{
    // ASCII value for '-' character.
    const uint8 ASCII_MINUS = 45;
    const uint8 MAX_BASE = 16;
    bool success = false;
    int len = s.length();
    int digitWeight = 1;
    
    value = 0;
        
    if (base <= MAX_BASE)
    {
        for (int i = 0; i < len; i++)
        {
            // Start with rightmost character.
            uint8 digitChar = s[len - 1 - i];
            uint8 digitValue;
            
            if ((i == len - 1) && (base == 10) && (digitChar == ASCII_MINUS))
            { // If first character is a '-' sign we support this by negating the value
                value *= -1;
            }
            else
            {
                // Convert the current digit to its integer representation
                success = DigitToInt(digitChar, digitValue);
                // Check against being valid with respect to base
                success = success && (digitValue < base);
                
                if (success)
                {
                    // Sum up value of this digit and step up digit weight
                    value += digitValue * digitWeight; 
                    digitWeight *= base;
                }
                else
                {
                    break;
                }
            }
        }
    }
    
    return success;
}


/*
    Convert the given character into its corresponding digit value.
    
    return true on success
*/
bool DigitToInt(uint8 digitChar, uint8 &out digitValue)
{
    const uint8 ASCII_0 = 48;
    const uint8 ASCII_9 = 57;
    const uint8 ASCII_A = 65;
    const uint8 ASCII_F = 70;
    const uint8 ASCII_a = 97;
    const uint8 ASCII_f = 102;
    
    bool success = false;
    
    if ((digitChar >= ASCII_0) && (digitChar <= ASCII_9))
    {
        digitValue = int(digitChar) -ASCII_0;
        success = true;   
    }
    else if ((digitChar >= ASCII_a) && (digitChar <= ASCII_f))
    {
        digitValue = int(digitChar) -ASCII_a + 10;
        success = true;
    }
    else if ((digitChar >= ASCII_A) && (digitChar <= ASCII_F))
    {
        digitValue = int(digitChar) -ASCII_A + 10;
        success = true;
    }
    
    return success;
}


